from datetime import datetime
from typing import Any, Literal

from pydantic import BaseModel, Field

from models.assess import StoryTag
from models.base import TaranisBaseModel


DashboardHealthState = Literal["up", "down", "n/a"]


class DashboardHealthServices(BaseModel):
    database: DashboardHealthState
    broker: DashboardHealthState
    workers: DashboardHealthState


class DashboardHealth(BaseModel):
    healthy: bool
    services: DashboardHealthServices


class Dashboard(TaranisBaseModel):
    _core_endpoint = "/dashboard"
    _model_name = "dashboard"
    _pretty_name = "Dashboard"
    _cache_timeout = 30
    total_news_items: int = 0
    total_story_items: int = 0
    total_products: int = 0
    report_items_completed: int = 0
    report_items_in_progress: int = 0
    latest_collected: datetime | None = None
    schedule_length: int | None = None
    conflict_count: int | None = None
    health_status: DashboardHealth | None = None
    worker_status: dict[str, dict[str, int]] | None = None


class TrendingCluster(TaranisBaseModel):
    _core_endpoint = "/dashboard/trending-clusters"
    _model_name = "trending_clusters"
    _pretty_name = "Trending Clusters"

    name: str
    tags: list[StoryTag] = Field(default_factory=list)
    size: int | None = None


class Cluster(TaranisBaseModel):
    _core_endpoint = "/dashboard/cluster"
    _model_name = "clusters"
    _pretty_name = "Clusters"

    name: str
    size: int | None = None


class StoryConflict(TaranisBaseModel):
    _core_endpoint = "connectors/conflicts/story-conflicts"
    _cache_timeout = 5
    _model_name = "story_conflicts"
    _pretty_name = "Story Conflicts"

    story_id: str
    existing_story: str
    incoming_story: str
    has_proposals: str | None = None


class NewsItemConflict(TaranisBaseModel):
    _core_endpoint = "/connectors/conflicts/news-items"
    _cache_timeout = 5
    _model_name = "news_item_conflict"
    _pretty_name = "News Item Conflicts"

    incoming_story_id: str
    news_item_id: str
    existing_story_id: str
    incoming_story: dict[str, Any]
    misp_address: str | None = None



from datetime import datetime
from typing import Self

from pydantic import Field, ValidationInfo, field_validator
from requests import Response

from models.base import TaranisBaseModel


class WorkerProduct(TaranisBaseModel):
    data: bytes | None = None
    mime_type: str | None = None

    @classmethod
    def from_response(cls, response: Response) -> Self:
        return cls(
            data=response.content,
            mime_type=response.headers.get("Content-Type", ""),
        )


class Product(TaranisBaseModel):
    _core_endpoint = "/publish/products"
    _model_name = "product"
    _pretty_name = "Product"

    id: str | None = None
    title: str
    description: str | None = ""
    created: datetime | None = None
    auto_publish: bool | None = None
    default_publisher: str | None = None
    product_type_id: int
    report_items: list[str] = Field(default_factory=list)
    last_rendered: datetime | None = None
    render_result: str | None = None
    mime_type: str | None = None

    @field_validator("default_publisher", mode="after")
    @classmethod
    def require_default_when_autopublish(cls, value: str, info: ValidationInfo):
        auto_publish = info.data.get("auto_publish")
        if auto_publish and not value:
            raise ValueError("default_publisher is required when auto_publish is enabled")
        return value



import contextlib
from enum import IntEnum, StrEnum, auto, nonmember


class TLPLevel(StrEnum):
    CLEAR = "clear"
    GREEN = "green"
    AMBER_STRICT = "amber+strict"
    AMBER = "amber"
    RED = "red"

    _ACCESSIBLE_NAMES = nonmember(
        {
            "RED": ["RED", "AMBER_STRICT", "AMBER", "GREEN", "CLEAR"],
            "AMBER_STRICT": ["AMBER_STRICT", "AMBER", "GREEN", "CLEAR"],
            "AMBER": ["AMBER", "GREEN", "CLEAR"],
            "GREEN": ["GREEN", "CLEAR"],
            "CLEAR": ["CLEAR"],
        }
    )

    def get_accessible_levels(self) -> list[str]:
        """
        Return the list of TLPLevel members this level can access.
        """
        names = type(self)._ACCESSIBLE_NAMES.get(self.name, [])
        return [type(self)[nm].value for nm in names]

    @classmethod
    def get_most_restrictive_tlp(cls, tlp_levels: list["TLPLevel"]) -> "TLPLevel":
        """
        Get the most restrictive TLP level from a list of TLP levels.
        If the list is empty, return the default TLP level (CLEAR).
        """
        if not tlp_levels:
            return cls.CLEAR

        provided = {tlp.name for tlp in tlp_levels}

        return next(
            (cls[level_name] for level_name in cls._ACCESSIBLE_NAMES.keys() if level_name in provided),
            cls.CLEAR,
        )

    @classmethod
    def get_tlp_level(cls, tlp_level: str) -> "TLPLevel | None":
        with contextlib.suppress(ValueError):
            return TLPLevel(tlp_level)
        return None


class ItemType(StrEnum):
    OSINT_SOURCE = auto()
    OSINT_SOURCE_GROUP = auto()
    WORD_LIST = auto()
    REPORT_ITEM_TYPE = auto()
    PRODUCT_TYPE = auto()

    @classmethod
    def _missing_(cls, value):
        if isinstance(value, str):
            for member in cls:
                if value.upper() == member.name:
                    return member
        return super()._missing_(value)


class COLLECTOR_TYPES(StrEnum):
    RSS_COLLECTOR = auto()
    EMAIL_COLLECTOR = auto()
    TWITTER_COLLECTOR = auto()
    SIMPLE_WEB_COLLECTOR = auto()
    RT_COLLECTOR = auto()
    MISP_COLLECTOR = auto()
    MANUAL_COLLECTOR = auto()
    PPN_COLLECTOR = auto()

    @classmethod
    def _missing_(cls, value):
        if isinstance(value, str):
            for member in cls:
                if value.upper() == member.name:
                    return member
        return super()._missing_(value)


class BOT_TYPES(StrEnum):
    ANALYST_BOT = auto()
    GROUPING_BOT = auto()
    NLP_BOT = auto()
    IOC_BOT = auto()
    TAGGING_BOT = auto()
    STORY_BOT = auto()
    SUMMARY_BOT = auto()
    WORDLIST_BOT = auto()
    SENTIMENT_ANALYSIS_BOT = auto()
    CYBERSEC_CLASSIFIER_BOT = auto()

    @classmethod
    def _missing_(cls, value):
        if isinstance(value, str):
            for member in cls:
                if value.upper() == member.name:
                    return member
        return super()._missing_(value)


class PRESENTER_TYPES(StrEnum):
    PANDOC_PRESENTER = auto()
    PDF_PRESENTER = auto()
    HTML_PRESENTER = auto()
    TEXT_PRESENTER = auto()
    JSON_PRESENTER = auto()
    STIX_PRESENTER = auto()

    @classmethod
    def _missing_(cls, value):
        if isinstance(value, str):
            for member in cls:
                if value.upper() == member.name:
                    return member
        return super()._missing_(value)


class PUBLISHER_TYPES(StrEnum):
    FTP_PUBLISHER = auto()
    SFTP_PUBLISHER = auto()
    S3_PUBLISHER = auto()
    EMAIL_PUBLISHER = auto()
    WORDPRESS_PUBLISHER = auto()
    MISP_PUBLISHER = auto()
    TAXII_PUBLISHER = auto()

    @classmethod
    def _missing_(cls, value):
        if isinstance(value, str):
            for member in cls:
                if value.upper() == member.name:
                    return member
        return super()._missing_(value)


class WORKER_TYPES(StrEnum):
    RSS_COLLECTOR = auto()
    EMAIL_COLLECTOR = auto()
    TWITTER_COLLECTOR = auto()
    SIMPLE_WEB_COLLECTOR = auto()
    MANUAL_COLLECTOR = auto()
    RT_COLLECTOR = auto()
    MISP_COLLECTOR = auto()
    ANALYST_BOT = auto()
    GROUPING_BOT = auto()
    NLP_BOT = auto()
    IOC_BOT = auto()
    TAGGING_BOT = auto()
    STORY_BOT = auto()
    SUMMARY_BOT = auto()
    SENTIMENT_ANALYSIS_BOT = auto()
    CYBERSEC_CLASSIFIER_BOT = auto()
    WORDLIST_BOT = auto()
    PANDOC_PRESENTER = auto()
    PDF_PRESENTER = auto()
    HTML_PRESENTER = auto()
    TEXT_PRESENTER = auto()
    JSON_PRESENTER = auto()
    STIX_PRESENTER = auto()
    FTP_PUBLISHER = auto()
    SFTP_PUBLISHER = auto()
    S3_PUBLISHER = auto()
    EMAIL_PUBLISHER = auto()
    WORDPRESS_PUBLISHER = auto()
    MISP_PUBLISHER = auto()
    TAXII_PUBLISHER = auto()
    MISP_CONNECTOR = auto()
    PPN_COLLECTOR = auto()

    @classmethod
    def _missing_(cls, value):
        if isinstance(value, str):
            for member in cls:
                if value.upper() == member.name:
                    return member
        return super()._missing_(value)


class CONNECTOR_TYPES(StrEnum):
    MISP_CONNECTOR = auto()

    @classmethod
    def _missing_(cls, value):
        if isinstance(value, str):
            for member in cls:
                if value.upper() == member.name:
                    return member
        return super()._missing_(value)


class WORKER_CATEGORY(StrEnum):
    COLLECTOR = auto()
    BOT = auto()
    PRESENTER = auto()
    PUBLISHER = auto()
    CONNECTOR = auto()

    @classmethod
    def _missing_(cls, value):
        if isinstance(value, str):
            for member in cls:
                if value.upper() == member.name:
                    return member
        return super()._missing_(value)


class WordListUsage(IntEnum):
    COLLECTOR_INCLUDELIST = 1  # 2^0
    COLLECTOR_EXCLUDELIST = 2  # 2^1
    TAGGING_BOT = 4  # 2^2


class OSINTState(IntEnum):
    DISABLED = -2
    UNKNOWN = -1
    OK = 0
    ERROR = 1
    NOT_MODIFIED = 2


class AttributeType(StrEnum):
    STRING = auto()
    NUMBER = auto()
    BOOLEAN = auto()
    RADIO = auto()
    ENUM = auto()
    TEXT = auto()
    RICH_TEXT = auto()
    DATE = auto()
    TIME = auto()
    DATE_TIME = auto()
    LINK = auto()
    ATTACHMENT = auto()
    TLP = auto()
    CPE = auto()
    CVE = auto()
    CVSS = auto()
    STORY = auto()

    @classmethod
    def _missing_(cls, value):
        if isinstance(value, str):
            for member in cls:
                if value.upper() == member.name:
                    return member
        return super()._missing_(value)



"""
This type stub file was generated by pyright.
"""

from datetime import datetime

from models.base import TaranisBaseModel

"""
This type stub file was generated by pyright.
"""

class WorkerProduct(TaranisBaseModel):
    data: bytes | None = ...
    mime_type: str | None = ...

class Product(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _search_fields = ...
    _pretty_name = ...
    id: str | None = ...
    title: str
    description: str | None = ...
    created: datetime | None = ...
    auto_publish: bool | None = ...
    product_type_id: int
    report_items: list[str] = ...
    last_rendered: datetime | None = ...
    render_result: str | None = ...
    mime_type: str | None = ...



from datetime import datetime
from typing import ClassVar

from pydantic import Field

from models.admin import ReportItemAttribute
from models.assess import Story
from models.base import TaranisBaseModel


class ReportItemCpe(TaranisBaseModel):
    id: int | None = None
    value: str


class ReportItemAttributeGroup(TaranisBaseModel):
    title: str
    attributes: list[ReportItemAttribute] = Field(default_factory=list)


class ReportTypes(TaranisBaseModel):
    _core_endpoint: ClassVar[str] = "/analyze/report-types"
    _model_name: ClassVar[str] = "report_type"
    _pretty_name: ClassVar[str] = "Report Type"

    id: int | None = None
    title: str | None = ""


class ReportItem(TaranisBaseModel):
    _core_endpoint: ClassVar[str] = "/analyze/report-items"
    _model_name: ClassVar[str] = "report"
    _pretty_name: ClassVar[str] = "Report"

    id: str | None = None
    title: str | None = ""
    created: datetime | None = None
    last_updated: datetime | None = None
    completed: bool | None = None
    user_id: int | None = None
    report_item_type_id: int | None = None
    report_item_type: str | None = None
    stories: list[Story | str] | None = None
    grouped_attributes: list[ReportItemAttributeGroup] | None = None
    attributes: list[ReportItemAttribute] | dict[str, str] = Field(default_factory=list)
    report_item_cpes: list[ReportItemCpe] | None = None



import contextlib
import hashlib
import re
from datetime import datetime, timezone
from typing import Annotated, Any, Literal, Self
from urllib.parse import quote

import language_tags
from bs4 import BeautifulSoup
from pydantic import BeforeValidator, ValidationInfo, field_validator, model_validator

from models.base import TaranisBaseModel


def _utcnow() -> datetime:
    return datetime.now(timezone.utc).replace(tzinfo=None)


def _normalize_datetime(value: str | datetime | None) -> datetime | None:
    if value is None:
        return None
    if isinstance(value, str):
        with contextlib.suppress(ValueError):
            value = datetime.fromisoformat(value)
        if isinstance(value, str):
            return None
    if value.tzinfo is None or value.utcoffset() is None:
        return value
    return value.astimezone(timezone.utc).replace(tzinfo=None)


def validate_bcp47(value: str | None) -> str | None:
    if value is None:
        return None
    if not isinstance(value, str):
        raise TypeError("BCP47 language tag must be a string")
    if not language_tags.tags.check(value):
        raise ValueError(f"Invalid BCP 47 language tag: {value}")
    return value


BCP47 = Annotated[str | None, BeforeValidator(validate_bcp47)]


class NewsItem(TaranisBaseModel):
    _core_endpoint = "/assess/news-items"

    osint_source_id: str
    id: str | None = None
    hash: str | None = None
    title: str | None = None
    author: str | None = None
    review: str | None = None
    content: str | None = None
    link: str | None = None
    source: str | None = None
    published: datetime | None = None
    collected: datetime | None = None
    updated: datetime | None = None
    attributes: list[str | dict[str, Any]] | None = None
    story_id: str | None = None
    language: BCP47 = None
    last_change: str | None = None

    @model_validator(mode="before")
    @classmethod
    def normalize_model_input(cls, data: Any) -> Any:
        if not isinstance(data, dict):
            return data

        normalized = dict(data)
        normalized.pop("updated", None)
        normalized.pop("hash", None)
        if normalized.get("language") in ("", None):
            normalized.pop("language", None)
        return normalized

    @classmethod
    def from_input(cls, data: Any) -> Self:
        """Untrusted input: run validators + sanitizers."""
        return cls.model_validate(data)

    @classmethod
    def from_db(cls, data: dict[str, Any]) -> Self:
        """Trusted DB row: skip *all* validators/sanitizers."""
        return cls.model_construct(**data)

    @classmethod
    def normalize_datetime(cls, date: str | datetime | None, default_to_now: bool = False) -> datetime | None:
        return _normalize_datetime(date) or (_utcnow() if default_to_now else None)

    @model_validator(mode="after")
    def check_required_fields(self) -> Self:
        if not self.title and not self.content:
            raise ValueError("Either title or content must be provided.")
        return self

    @field_validator("title", "content", mode="before")
    @classmethod
    def sanitize_html(cls, value: str, info: ValidationInfo) -> str:
        if not value:
            return ""
        html = re.sub(r"(?i)(&nbsp;|\xa0)", " ", value, re.DOTALL)
        return BeautifulSoup(html, "lxml").text

    @field_validator("link", mode="before")
    @classmethod
    def sanitize_url(cls, url: str, info: ValidationInfo) -> str:
        return quote(url or "", safe="/:@?&=+$,;")

    @model_validator(mode="after")
    def ensure_hash(self) -> Self:
        if not self.hash:
            title = self.title or ""
            link = self.link or ""
            self.hash = hashlib.sha256((title + link).encode()).hexdigest()
        return self

    @field_validator("published", "collected", mode="before")
    @classmethod
    def sanitize_date(cls, date: str | None | datetime) -> datetime:
        return cls.normalize_datetime(date, default_to_now=True) or _utcnow()

    def to_core_dict(self) -> dict[str, Any]:
        return self.model_dump(exclude={"updated"})


class StoryTag(TaranisBaseModel):
    name: str
    size: int
    type: str | None = None


class Story(TaranisBaseModel):
    _core_endpoint = "/assess/stories"
    _model_name = "story"
    _pretty_name = "Story"
    _cache_timeout = 30

    id: str | None = None
    title: str | None = None
    description: str | None = None
    created: datetime | None = None
    updated: datetime | None = None
    last_change: str | None = None
    news_items: list[NewsItem] | None = None
    links: list[str] | None = None
    important: bool | None = None
    read: bool | None = None
    likes: int | None = None
    dislikes: int | None = None
    user_vote: Literal["like", "dislike", "", None] = None
    summary: str | None = None
    relevance: int | None = None
    relevance_override: int | None = None
    comments: str | None = None
    in_reports_count: int | None = None
    tags: list[dict[str, Any]] | None = None
    attributes: list[dict[str, Any]] | None = None

    @classmethod
    def normalize_datetime(cls, date: str | datetime | None) -> datetime | None:
        return _normalize_datetime(date)

    @field_validator("created", "updated", mode="before")
    @classmethod
    def sanitize_story_dates(cls, date: str | None | datetime) -> datetime | None:
        return cls.normalize_datetime(date)


class AssessSource(TaranisBaseModel):
    id: str | None = None
    icon: str | None = None
    name: str
    type: str | None = None


class FilterLists(TaranisBaseModel):
    _core_endpoint = "/assess/filterlists"
    _model_name = "filter_lists"
    _pretty_name = "Filter Lists"

    tags: list[str] = []
    sources: list[AssessSource] = []
    groups: list[dict[str, Any]] = []


class StoryUpdatePayload(TaranisBaseModel):
    vote: Literal["like", "dislike", ""] | None = None
    important: bool | None = None
    read: bool | None = None
    title: str | None = None
    description: str | None = None
    comments: str | None = None
    summary: str | None = None
    tags: list[dict[str, Any]] | None = None
    attributes: list[dict[str, Any]] | None = None


class BulkAction(TaranisBaseModel):
    story_ids: list[str] = []
    payload: StoryUpdatePayload | None = None



from typing import ClassVar, TypeVar

from pydantic import BaseModel, ConfigDict, ValidationError


T = TypeVar("T", bound="TaranisBaseModel")


class TaranisBaseModel(BaseModel):
    _core_endpoint: ClassVar[str]
    _cache_timeout: ClassVar[int]
    _model_name: ClassVar[str] = ""
    _pretty_name: ClassVar[str] = ""

    model_config = ConfigDict(extra="allow")

    @classmethod
    def extract_validation_errors(cls, exc: ValidationError) -> list[dict[str, str]]:
        return [
            {
                "model": cls.__name__,
                "field": ".".join(str(loc) for loc in error["loc"]),
                "error": error["msg"],
                "error_type": error["type"],
            }
            for error in exc.errors()
        ]

    @classmethod
    def format_validation_errors(cls, exc: ValidationError) -> str:
        structured = cls.extract_validation_errors(exc)
        if not structured:
            return f"Validation failed for model {cls.__name__}, but no error details were extracted."

        lines = [f"Validation failed for model '{cls.__name__}':"]
        lines.extend(f"  • Field '{err['field']}': {err['error']} (type={err['error_type']})" for err in structured)
        return "\n".join(lines)

    @classmethod
    def validation_error_response(cls, exc: ValidationError, prefix: str) -> dict[str, str]:
        errors = "; ".join(f"{err['field']}: {err['error']}" for err in cls.extract_validation_errors(exc))
        return {"error": f"{prefix}{f': {errors}' if errors else ''}"}

    def model_dump(self, *args, **kwargs):
        kwargs.setdefault("exclude_none", True)
        kwargs.setdefault("exclude_defaults", False)
        kwargs.setdefault("exclude_unset", False)
        return super().model_dump(*args, **kwargs)



from pydantic import Field

from models.base import TaranisBaseModel


class Asset(TaranisBaseModel):
    _core_endpoint = "/assets"
    _model_name = "asset"
    _pretty_name = "Assets"

    id: int | None = None
    name: str | None = ""
    description: str | None = ""
    serial: str | None = ""
    asset_group_id: str | None = None
    asset_cpes: list[int] = Field(default_factory=list)
    vulnerabilities: list[int] = Field(default_factory=list)
    vulnerabilities_count: int | None = None



"""
This type stub file was generated by pyright.
"""

from typing import Any

from models.base import TaranisBaseModel

"""
This type stub file was generated by pyright.
"""

class ProfileSettingsDashboard(TaranisBaseModel):
    show_trending_clusters: bool = ...
    show_charts: bool = ...
    trending_cluster_days: int = ...
    trending_cluster_filter: list[Any] = ...

class ProfileSettings(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    dark_theme: bool = ...
    compact_view: bool = ...
    show_charts: bool = ...
    infinite_scroll: bool = ...
    advanced_story_options: bool = ...
    language: str = ...
    hotkeys: dict[str, Any] = ...
    split_view: bool = ...
    end_of_shift: str | None = ...
    highlight: bool = ...
    dashboard: ProfileSettingsDashboard = ...

class UserProfile(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _search_fields = ...
    id: int | None = ...
    username: str = ...
    name: str
    organization: dict[str, Any] | None = ...
    profile: ProfileSettings = ...
    permissions: list[str] | None = ...
    roles: list[dict[str, Any]] | None = ...



"""
This type stub file was generated by pyright.
"""

from datetime import datetime
from typing import ClassVar, Final

from models.admin import ReportItemAttribute
from models.assess import Story
from models.base import TaranisBaseModel

"""
This type stub file was generated by pyright.
"""

class ReportItemCpe(TaranisBaseModel):
    id: int | None = ...
    value: str

class ReportItemAttributeGroup(TaranisBaseModel):
    title: str
    attributes: list[ReportItemAttribute] = ...

class ReportItem(TaranisBaseModel):
    _core_endpoint: ClassVar[str] = ...
    _model_name: ClassVar[str] = ...
    _search_fields: Final[list[str]] = ...
    _pretty_name: ClassVar[str] = ...
    id: str | None = ...
    title: str | None = ...
    created: datetime | None = ...
    last_updated: datetime | None = ...
    completed: bool | None = ...
    user_id: int | None = ...
    report_item_type_id: int | None = ...
    report_item_type: str | None = ...
    stories: list[Story | str] | None = ...
    grouped_attributes: list[ReportItemAttributeGroup] | None = ...
    attributes: list[ReportItemAttribute] | dict[str, str] = ...
    report_item_cpes: list[ReportItemCpe] | None = ...



"""
This type stub file was generated by pyright.
"""

from models.base import TaranisBaseModel

"""
This type stub file was generated by pyright.
"""

class Asset(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _search_fields = ...
    _pretty_name = ...
    id: int | None = ...
    name: str | None = ...
    description: str | None = ...
    serial: str | None = ...
    asset_group_id: str | None = ...
    asset_cpes: list[int] = ...
    vulnerabilities: list[int] = ...
    vulnerabilities_count: int | None = ...



"""
This type stub file was generated by pyright.
"""

from datetime import datetime
from functools import cached_property
from typing import Any, Literal

from pydantic import AnyUrl

from models.base import TaranisBaseModel
from models.types import (
    BOT_TYPES,
    COLLECTOR_TYPES,
    CONNECTOR_TYPES,
    PRESENTER_TYPES,
    PUBLISHER_TYPES,
    WORKER_CATEGORY,
    WORKER_TYPES,
    AttributeType,
    ItemType,
    TLPLevel,
)

"""
This type stub file was generated by pyright.
"""

class Job(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    id: str | None = ...
    name: str
    trigger: str | None = ...
    kwargs: str | None = ...
    next_run_time: str | None = ...

class ExportStoriesQuery(TaranisBaseModel):
    timefrom: datetime | None = ...
    timeto: datetime | None = ...
    metadata: bool = ...

class TaskResult(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    id: str | None = ...
    result: Any | None = ...
    status: str | None = ...
    last_change: datetime | None = ...
    last_success: datetime | None = ...
    def __eq__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...

class Address(TaranisBaseModel):
    city: str = ...
    country: str = ...
    street: str = ...
    zip: str = ...

class Organization(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _search_fields = ...
    id: int | None = ...
    name: str = ...
    description: str | None = ...
    address: Address = ...

class Permission(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    id: str | None = ...
    name: str
    description: str = ...

class ACL(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _search_fields = ...
    _pretty_name = ...
    id: int | None = ...
    name: str = ...
    description: str | None = ...
    item_type: ItemType | None = ...
    item_id: str | None = ...
    roles: list[int] = ...
    read_only: bool = ...
    enabled: bool = ...

class ParameterValue(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    id: int | None = ...
    parameter: str = ...
    value: str | None = ...

class Worker(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _search_fields = ...
    _pretty_name = ...
    id: str | None = ...
    name: str
    description: str | None = ...
    type: WORKER_TYPES
    category: WORKER_CATEGORY
    parameters: dict[str, str] = ...

class Role(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _search_fields = ...
    _pretty_name = ...
    id: int | None = ...
    name: str = ...
    description: str | None = ...
    permissions: list[str] = ...
    tlp_level: TLPLevel | None = ...

class User(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _search_fields = ...
    id: int | None = ...
    name: str
    organization: dict | int
    permissions: list[str] | None = ...
    profile: dict | None = ...
    roles: list[int] | list[dict] = ...
    username: str = ...
    password: str | None = ...

class TaranisConfig(TaranisBaseModel):
    default_collector_proxy: AnyUrl | Literal[""] = ...
    default_collector_interval: str = ...
    default_tlp_level: TLPLevel = ...
    default_story_conflict_retention: str = ...
    default_news_item_conflict_retention: str = ...

class Settings(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    _cache_timeout = ...
    id: int = ...
    settings: TaranisConfig = ...

class WordListEntry(TaranisBaseModel):
    value: str
    category: str | None = ...
    description: str | None = ...

class WordList(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    _search_fields = ...
    id: int | None = ...
    name: str
    description: str | None = ...
    usage: list[str] = ...
    link: str = ...
    entries: list[WordListEntry] | None = ...

class OSINTSource(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    id: str | None = ...
    name: str
    description: str = ...
    rank: int = ...
    type: COLLECTOR_TYPES | None = ...
    parameters: dict[str, str] | None = ...
    icon: str | None = ...
    enabled: bool | None = ...
    status: TaskResult | None = ...
    @cached_property
    def search_field(self) -> str: ...

class OSINTSourceGroup(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    _search_fields = ...
    id: str | None = ...
    name: str
    description: str = ...
    default: bool = ...
    osint_sources: list[str] = ...
    word_lists: list[int] = ...

class ProductParameterValue(TaranisBaseModel):
    TEMPLATE_PATH: str | None = ...

class ProductType(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    _search_fields = ...
    id: int | None = ...
    title: str
    description: str = ...
    type: PRESENTER_TYPES
    parameters: ProductParameterValue = ...
    report_types: list[int] = ...
    status: TaskResult | None = ...

class PublisherPreset(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    _search_fields = ...
    id: str | None = ...
    name: str
    type: PUBLISHER_TYPES
    description: str | None = ...
    parameters: dict[str, str] | None = ...
    status: TaskResult | None = ...

class ReportItemAttribute(TaranisBaseModel):
    id: int | None = ...
    attribute: dict[str, str] | None = ...
    attribute_id: int | None = ...
    attribute_group_id: int | None = ...
    title: str | None = ...
    description: str | None = ...
    index: int | None = ...
    required: bool | None = ...
    type: str | None = ...

class ReportItemAttributeGroup(TaranisBaseModel):
    id: int | None = ...
    title: str | None = ...
    description: str | None = ...
    index: int | None = ...
    attribute_group_items: list[ReportItemAttribute] = ...

class ReportItemType(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    _search_fields = ...
    id: int | None = ...
    title: str
    description: str = ...
    attribute_groups: list[ReportItemAttributeGroup] | None = ...

class Template(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    id: str
    content: str | None = ...
    validation_status: dict | None = ...

class AttributeEnum(TaranisBaseModel):
    id: int | None = ...
    index: int | None = ...
    value: str | None = ...
    description: str | None = ...
    imported: bool | None = ...
    attribute_id: int | None = ...

class Attribute(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    _search_fields = ...
    id: int | None = ...
    name: str
    description: str = ...
    type: AttributeType
    default_value: str = ...
    attribute_enums: list[AttributeEnum] = ...

class Bot(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    _search_fields = ...
    id: str | None = ...
    name: str
    description: str = ...
    type: BOT_TYPES
    index: int | None = ...
    parameters: dict[str, str] | None = ...
    status: TaskResult | None = ...

class Connector(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    _search_fields = ...
    id: str | None = ...
    name: str
    description: str = ...
    type: CONNECTOR_TYPES | None = ...
    index: int | None = ...
    parameters: dict[str, str] | None = ...
    icon: str | None = ...
    status: TaskResult | None = ...

class WorkerParameterValue(TaranisBaseModel):
    name: str
    label: str | None = ...
    parent: Literal["parameters"] = ...
    type: str | None = ...
    rules: list[str] = ...

class WorkerParameter(TaranisBaseModel):
    _core_endpoint = ...
    _model_name = ...
    _pretty_name = ...
    id: str
    parameters: list[WorkerParameterValue]
